/* ========================================
 *
 * Copyright YOUR COMPANY, THE YEAR
 * All Rights Reserved
 * UNPUBLISHED, LICENSED SOFTWARE.
 *
 * CONFIDENTIAL AND PROPRIETARY INFORMATION
 * WHICH IS THE PROPERTY OF your company.
 *
 * ========================================
*/
#include <project.h>
#include "stdio_user.h"
#include <stdio.h>

#if CY_BLE_HOST_CORE

/*******************************************************************************
* Macros
*******************************************************************************/

#define ENABLE      (1u)
#define DISABLE     (0u)

#define DEBUG_BLE_ENABLE    DISABLE

#if DEBUG_BLE_ENABLE
#define DEBUG_BLE       printf
#else
#define DEBUG_BLE(...)
#endif


/*******************************************************************************
* Variables
*******************************************************************************/
cy_en_ble_api_result_t apiResult;
uint8 i;

/*******************************************************************************
*        Function Prototypes
*******************************************************************************/
void Ble_Init(void);
void StartScan(void);
void StackEventHandler(uint32 event, void* eventParam);
void BleAssert(void);

/*******************************************************************************
* Function Name: HostMain()
********************************************************************************
* Summary:
*  Main function for the BLE host.
*
* Parameters:
*  None
*
* Return:
*  None
*
* Theory:
*  The function starts BLE and UART components.
*  This function processes all BLE events
*
*******************************************************************************/
int HostMain(void)
{
    __enable_irq(); /* Enable global interrupts. */
       
	UART_Start();
    /* \x1b[2J\x1b[;H - ANSI ESC sequence for clear screen */
    printf("\x1b[2J\x1b[;H");
   
    printf("***************** PSoC6 BLE Observer project *******"\
               "**********\r\n");
   
    /* Initialize BLE */
    Ble_Init();

    for(;;) 
    {   
      /* Cy_BLE_ProcessEvents() allows BLE stack to process pending events */
       Cy_BLE_ProcessEvents();
    }
} 

/*******************************************************************************
* Function Name: Ble_Init()
********************************************************************************
*
* Summary:
*   This function initializes BLE.
*
* Return:
*   None
*
*******************************************************************************/
void Ble_Init(void)
{
    cy_en_ble_api_result_t          apiResult;
    cy_stc_ble_stack_lib_version_t  stackVersion;    
    
    apiResult = Cy_BLE_GetStackLibraryVersion(&stackVersion);
    
    if(apiResult != CY_BLE_SUCCESS)
    {
        DEBUG_BLE("Cy_BLE_GetStackLibraryVersion API Error: 0x%2.2x \r\n",\
                   apiResult);
    }
    else
    {
        DEBUG_BLE("Stack Library Version: Major Version-%u, Minor Version-%u,"\
        "Patch No-%u, Build No-%u\r\n", stackVersion.majorVersion,\
        stackVersion.minorVersion, stackVersion.patch, stackVersion.buildNumber);
    }
    
    printf("*****************************************************************"\
               "*****************\r\n\n");
    
     /* Start BLE component and register generic event handler */
    apiResult = Cy_BLE_Start(StackEventHandler);
    
    if(apiResult != CY_BLE_SUCCESS)
    {
        /* BLE stack initialization failed, check configuration 
           and halt CPU in debug mode */
        DEBUG_BLE("Cy_BLE_Start API Error: %x \r\n", apiResult);
        BleAssert();    
    }
    else
    {
        DEBUG_BLE("BLE Stack Initialized..\r\n");
    }
}

/*******************************************************************************
* Function Name: StackEventHandler()
********************************************************************************
*
* Summary:
*   This is an event callback function to receive events from the BLE Component.
*
*  event - the event code
*  *eventParam - the event parameters
*
* Return:
*   None
*
*******************************************************************************/
void StackEventHandler(uint32 event, void *eventParam)
{
    cy_stc_ble_gapc_adv_report_param_t * advReport;
   
    uint8_t scanState = Cy_BLE_GetScanState();
    
    switch (event)
	{
        /* This event is received when the BLE stack is initialized and turned ON.*/
        case CY_BLE_EVT_STACK_ON:
        {
            DEBUG_BLE("CY_BLE_EVT_STACK_ON \r\n");    
            /* Stack initialized; ready for scan */
              StartScan();
            
            break;
        }

        case CY_BLE_EVT_GAPC_SCAN_PROGRESS_RESULT:  
                
            /* scan progress result event occurs 
             * when it receives any advertisiment packet
             * or scan response packet from peer device*/

            advReport = (cy_stc_ble_gapc_adv_report_param_t *)eventParam;
      
            printf("CYBLE_EVT_GAPC_SCAN_PROGRESS_RESULT:\r\n");
             
            /* Print the Advertising Event details of peer device:*/
            printf("eventType:");
            
            switch(advReport->eventType)
            {
                case CY_BLE_GAPC_CONN_UNDIRECTED_ADV: 
                    
                     printf("Connectable undirected advertising\r\n");
                    break;
                    
                case  CY_BLE_GAPC_CONN_DIRECTED_ADV:
                     
                     printf("Connectable directed advertising\r\n");
                    break;
                
                case  CY_BLE_GAPC_SCAN_UNDIRECTED_ADV:
                    
                     printf("Scannable undirected advertising\r\n");
                    break;
                 
                case CY_BLE_GAPC_NON_CONN_UNDIRECTED_ADV:
                    
                      printf("Non connectable undirected advertising\r\n");
                    break;
                 
                case CY_BLE_GAPC_SCAN_RSP:
                    
                      printf("SCAN_RSP\r\n");
                    break;
            }
            /*  PEER addr type  */
            printf("    peerAddrType: ");
            
            if(advReport->peerAddrType==CY_BLE_GAP_ADDR_TYPE_PUBLIC)
            {
                printf("PUBLIC\r\n");
            }
            else if(advReport->peerAddrType==CY_BLE_GAP_ADDR_TYPE_PUBLIC)
            {
                printf("RANDOM \r\n");
            }
            
             /*  PEER Device address type  */
            printf("    peerBdAddr: %2.2x%2.2x%2.2x%2.2x%2.2x%2.2x \r\n",
                advReport->peerBdAddr[5u], advReport->peerBdAddr[4u],
                advReport->peerBdAddr[3u], advReport->peerBdAddr[2u],
                advReport->peerBdAddr[1u], advReport->peerBdAddr[0u]);
            
            /*  Advertising or scan response data  */
            printf("    Peer device adveritsing/scan response data Length: %x \r\n", advReport->dataLen);
            printf("    advertising/scan response data of peer device: ");

            
            if(advReport->dataLen!=0)
            {
                for(i = 0u; i < advReport->dataLen; i++)
                {
                    printf("%x", advReport->data[advReport->dataLen-i]);
                }
            
            }
            printf("\r\n");
            
            /*  RSSI of the received packet from peer Device  */
            printf("    Rssi: %i \r\n", advReport->rssi);
            
            printf("\r\n");
        break;  
      
        /* This event is triggered when the central device has started/stopped scanning */
        case CY_BLE_EVT_GAPC_SCAN_START_STOP:
        {
            DEBUG_BLE("CY_BLE_EVT_GAPC_SCAN_START_STOP, scanstate = ");
            switch(scanState)
            {
                case CY_BLE_SCAN_STATE_STOPPED:
                    DEBUG_BLE("CY_BLE_SCAN_STATE_STOPPED\r\n");
                    break;
                
                case CY_BLE_SCAN_STATE_SCAN_INITIATED:
                    DEBUG_BLE("CY_BLE_SCAN_STATE_SCAN_INITIATED\r\n");
                    break;
                    
                case CY_BLE_SCAN_STATE_SCANNING:
                    DEBUG_BLE("CY_BLE_SCAN_STATE_SCANNING\r\n");
                    break;
                    
                case CY_BLE_SCAN_STATE_STOP_INITIATED:
                    DEBUG_BLE("CY_BLE_SCAN_STATE_STOP_INITIATED\r\n");
                    break;
            }
           
            break;
        }    
      
        default:
         	break;
	}
}

/*******************************************************************************
* Function Name: StartScan()
********************************************************************************
* Summary:
* Starts Scanning.
*
*******************************************************************************/
void StartScan(void)
{
    Cy_BLE_GAPC_StartScan(CY_BLE_SCANNING_FAST,\
                          CY_BLE_OBSERVER_CONFIGURATION_0_INDEX);
 
}

/*******************************************************************************
* Function Name: BleAssert()
********************************************************************************
* Summary:
*
* Parameters:
* None
*
* Return:
* None
*
*******************************************************************************/
void BleAssert(void)
{
    CY_ASSERT(0u != 0u);   
}

#endif /* CY_BLE_HOST_CORE */

/* [] END OF FILE */
